CON

  _CLKMODE      = XTAL1 + PLL16X                        
  _XINFREQ      = 5_000_000

  'Parallax Serial Terminal Control Character Constants
  HM =  1 'HM: Home Cursor       
  PC =  2 'PC: Position Cursor In X, Y
  ML =  3 'ML: Move Cursor Left          
  MR =  4 'MR: Move Cursor Right         
  MU =  5 'MU: Move Cursor Up          
  MD =  6 'MD: Move Cursor Down
  BP =  7 'BP: Beep Speaker
  BS =  8 'BS: Backspace
  TB =  9 'TB: Tab
  LF = 10 'LF: Line Feed
  CE = 11 'CE: Clear To End Of line     
  CB = 12 'CB: Clear Lines Below
  CR = 13 'CR: Carriage Return          
  PX = 14 'PX: Position Cursor In X         
  PY = 15 'PY: Position Cursor In Y
  CS = 16 'CS: Clear Screen
  SP = 32 'Space

  'Master And Slave Communication Constants
  MASTER_RX = 0 'Master RX Pin
  MASTER_TX = 1 'Master TX Pin
  SLAVE_RX = MASTER_TX 'Slave RX Pin
  SLAVE_TX = MASTER_RX 'Slave TX Pin
  MASTER_COMM_MODE = 0
  SLAVE_COMM_MODE = 0
  COMM_BAUD_RATE = 115_200

  MAX_LEN_NUM_STR = 11 'For NumericString 10 Characters + Zero Terminator
  MAX_LEN_TX_STR = 22 'For NewTxString 21 Characters + Zero Terminator

  #0, Master, Slave
  
VAR

  LONG DataField1
  LONG DataField2
  LONG LongValue1  
  LONG LongValue2

  BYTE NumericString[MAX_LEN_NUM_STR]
  BYTE NewTxString[MAX_LEN_TX_STR]

OBJ
  Serial : "FullDuplexSerial"
  com[2] : "InterComm"

PUB Main

  Serial.Start(31, 30, 0, 115_200) 
  WAITCNT(CLKFREQ * 2 + CNT)

  Serial.Rxflush
  Serial.Str(@prolog)

  IFNOT com[Master].Start(MASTER_RX, MASTER_TX, MASTER_COMM_MODE, COMM_BAUD_RATE)
    Serial.Str(@setupM)
    ABORT

  IFNOT com[Slave].Start(SLAVE_RX, SLAVE_TX, SLAVE_COMM_MODE, COMM_BAUD_RATE)
    Serial.Str(@setupS)
    ABORT

  WAITCNT(CLKFREQ * 2 + CNT)

  ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  transfer(Master, Slave, 1, 2)

  ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  transfer(Slave, Master, 3, 4)

  ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  transfer(Master, Slave, 5, 6)

  ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  transfer(Slave, Master, POSX, POSX)

  ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  Serial.Str(@epilog)
  
DAT

prolog  byte    CS, "###### The InterComm Object Demonstration ######", CR, CR
        byte    "Featuring A Propeller To Propeller Communication Simulation", CR, CR
        byte    "Better, Faster, Stronger, More Dependable!", CR
        byte    "And Easier To Interface!", CR, CR, 0

epilog  byte    "Latest Revision: Handshaking Has Been Implemented!", CR, CR
        byte    "Hold Onto Your Bootstraps Boys And Girls!", CR
        byte    "The Next Revision Is FIFO Control!", CR, CR
        byte    "The InterComm Object Demonstration Is Now Complete!", 0

setupM  byte    "Master Initialization Failed", CR, 0
setupS  byte    "Slave Initialization Failed", CR, 0

xfer0a  byte    "The ", 0
xfer0b  byte    " Is Now Sending Two Long Values To The ", 0
xfer0c  byte    "!", CR, 0

xfer1   byte    CR, "Results Of The Transmission: "
        byte    CR, "Value For DataField1: ", 0
xfer2   byte    CR, "Value For DataField2: ", 0

node_M  byte    "Master", 0
node_S  byte    "Slave", 0

ID      word    @node_M, @node_S

PRI transfer(src, dst, left, right) | value1, value2

  serial.str(@xfer0a)
  serial.str(@@ID[src])
  serial.str(@xfer0b)
  serial.str(@@ID[dst])
  serial.str(@xfer0c)

  ConvertLongsIntoTxString(left, right)
  com[src].UpdateTxString(@NewTxString)

  REPEAT UNTIL com[dst].IsNewDataAvailable              ' auto propagation (!0 -> TRUE)

  com[dst].GetDataFields(@value1, @value2)

  serial.str(@xfer1)
  serial.dec(value1)
  serial.str(@xfer2)
  serial.dec(value2)

  serial.tx(CR)
  serial.tx(CR)

PRI ConvertLongsIntoTxString(Value1, Value2) | length, source

  length := strsize(source := ConvertDecimalToString(Value1))

  BYTEMOVE(@NewTxString, source, MAX_LEN_NUM_STR + 1)
  NewTxString[length++] := ":"
  BYTEMOVE(@NewTxString[length], ConvertDecimalToString(Value2), MAX_LEN_NUM_STR + 1)

PRI ConvertDecimalToString(DecimalValue) | Index, Divisor, ZeroPad   

  BYTEFILL(@NumericString, 0, MAX_LEN_NUM_STR)

  Index := 0  

  Divisor := 1_000_000_000
  ZeroPad~

  REPEAT 10
  
    IF (DecimalValue => Divisor)
    
      NumericString[Index++] := (DecimalValue / Divisor + "0")
      DecimalValue //= Divisor
      ZeroPad~~
      
    ELSEIF ZeroPad or (Divisor == 1)
    
      NumericString[Index++] := "0"
    Divisor /= 10 

  return @NumericString

DAT